=begin

== Name

eachdelegator.rb - enumable capability delegation library

Version 0.1.2

Copyright(c) 2001
Okada Jun
yun@nekome.net
http://www.nekome.net/


== Detail

  EachDelegator add Enumerable capability to existing method,
 which is iterator (takes block as argument).

  EachDelegator redefine original method to return
 EnumerableDelegator object when block has not given.
  EnumerableDelegator is an Enumerable Object,
 which delegate whose each method to original method.

  after required this file, all modules' instance method
 which named 'each_*' or '*_each' is automatically redefined.

  to add Enumerable capability to your own method, 
 which is not named 'each_*' or '*_each' , use Module#enum_delegator manually.


== Example

  require 'eachdelegator'

  class Array
    def all_pair                 # method not named 'each_*' or '*_each'
      for i in 0...(size - 1)
        yield at(i), at(i + 1)
      end
      self
    end

    each_delegator :all_pair     # redifine method manually,
  end

  p [1, 3, 5, 7, 13].all_pair.collect{|a, b| b - a}
   => [ 2, 2, 2, 6]

=end


class EachDelegator
  include Enumerable

  def initialize(method, *args)
    @method = method
    @args = args
  end

  def each(&block)
    @method.call(*@args, &block)
  end
end


class Module
  @@in_each_delegator = false

  def each_delegator(*args)
    @@in_each_delegator = true
    args.each do |func|
      func_old = "#{func}_each_delegator_old"
      module_eval <<-"END"
        alias #{func_old} #{func}

        def #{func}(*args, &block)
	  if block
	    #{func_old} *args, &block
	  else
	    EachDelegator.new(self.method(:#{func_old}), *args)
	  end
        end
      END
    end
    @@in_each_delegator = false
  end
end


ObjectSpace::each_object(Module) do |mod|
  next if mod === EachDelegator
  mod.each_delegator(*mod.instance_methods.grep(/^each_|_each$/))
  class << mod
    each_delegator(*instance_methods.grep(/^each_|_each$/))
  end
end


class Module
  private :each_delegator

  alias method_added_each_delegator_old method_added

  def method_added(name)
    method_added_each_delegator_old(name)
    if ! @@in_each_delegator && /^each_|_each$/ =~ name.id2name
      each_delegator name
    end
  end
end


if __FILE__ == $0
  sample = 'p "hoge".each_byte.each_with_index.collect{|a, i| [ i, a ]}'
  print sample, "\n", '=> '
  eval sample
end


=begin

== History

=== 2000/1/23 version 0.1.3
rename enumdelegator to eachdelegator

=== 2000/1/22 version 0.1.2

Module#method_added method was hooked to
automatically catch each_* methods' definition.

=== 2000/1/21 version 0.1.1

EnumDelegator, now can handle a method which takes arguments.

=== 2001/1/20 version 0.1.0

first release.


=end
